#ifndef GOAP_ACTION_HPP
#define GOAP_ACTION_HPP

#include "actor_world_state.hpp"
#include "state.hpp"
#include <godot_cpp/variant/string.hpp>
#include <godot_cpp/templates/vector.hpp>
#include <godot_cpp/templates/hash_map.hpp>

#define GOAP_ACTION(Name_) \
public: \
    _FORCE_INLINE_ static gd::String get_static_class() { return #Name_; }\
    _FORCE_INLINE_ gd::String get_class() const { return #Name_; }\
private:

namespace goap {
typedef int ActionID;

/*! A goap action construct.
 *
 * Child classes an be registered with `ActionDB::register_action`
 */
class Action {
    friend class ActionDB;
    GOAP_ACTION(Action);
public:
    virtual ~Action() = default;

    virtual bool is_possible_for(ActorWorldState *state) = 0;
    virtual bool is_done_for(ActorWorldState *state) = 0;
    virtual State *get_apply_state() const = 0;

    WorldState const &get_required() const;
    WorldState const &get_effects() const;

    ActionID get_id() const;
protected:
    Action() = default;
    Action(WorldState &requires, WorldState &effects);
private:
    WorldState requires{};
    WorldState effects{};
    ActionID id{-1};
};

/* Storage wrapper for managed action class.
 */
struct ActionData {
    ActionData() = default;
    ActionData(Action *action);
    ~ActionData();
    Action *data{nullptr};
};
}

class TestAction : public goap::Action {
    GOAP_ACTION(TestAction);
public:
    TestAction() = default;
    virtual bool is_done_for(goap::ActorWorldState *) override;
    virtual bool is_possible_for(goap::ActorWorldState *) override;
    virtual goap::State *get_apply_state() const override;
};

#endif // !GOAP_ACTION_HPP
